/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Basic functions                   */
/*                                                                    */
/* This sample code shows how to establish a session to a Workflow    */
/* server and query details about the logged-on user.                 */
/* Additionally, it can be used to change a user's password, handle   */
/* its substitute and toggle its absence status.                      */
/**********************************************************************/

#include <fmctjinc.hxx>                 // compiler-dependent #includes
#include <fmcjprun.hxx>                 // MQWF C++ API interface
#include <ctype.h>                      // character classification


int main (int argc, char ** argv)
{
  string                        systemGroupName         = "";
  string                        systemName              = "";
  string                        user                    = "";
  string                        password                = "";

  unsigned long                 c, i                    = 0;
  int                           usrchoice               = 0;
  string                        choice                  = "";

  FmcjExecutionService          * service;

  FmcjService::SessionMode      sessionMode = FmcjService::Default;

/*--------------------------------------------------------------------*/
/* Check command line parameters                                      */
/*--------------------------------------------------------------------*/
  if ( (argc<3) || (argc>6) )
  {
    cout << "\nUsage: " << argv[0]
         << " [-p|-pf] <UserID> <Password> [System Group] [System Name]\n\n"
         << "       -p : Session mode 'present'\n"
         << "       -pf: Force logon for session mode 'present'\n"
         << endl;
    return -1;
  }



  if (strncmp(argv[1],"-",1)==0)   // Check for session mode option
  {
    if (strncmp(argv[1],"-pf",3)==0)
      sessionMode=FmcjService::PresentHere;
    else
      if (strncmp(argv[1],"-p",2)==0)
        sessionMode=FmcjService::Present;
       else
         cout << "\nIgnoring unknown option: " << argv[1] << endl;

    user     = argv[2];
    password = argv[3];
    if (argc > 4)
      systemGroupName=argv[4];
    if (argc > 5)
      systemGroupName=argv[5];
  }
  else                             // Use default session mode
  {
    user     = argv[1];
    password = argv[2];
    if (argc > 3)
      systemGroupName=argv[3];
    if (argc > 4)
      systemGroupName=argv[4];
  }

/*-------------------------------------------------------------------*/
/* Init API environment. Invoke FmcjGlobalConnect() in every thread  */
/* before the server is called.                                      */
/*-------------------------------------------------------------------*/
  APIRET rc = FmcjGlobal::Connect();

/*-------------------------------------------------------------------*/
/* Fetch result object for current thread                            */
/* Allocate ExecutionService for the given system parameters         */
/*-------------------------------------------------------------------*/
  FmcjResult * result = FmcjResult::ObjectOfCurrentThread();

  // Choose the correct constructor depending on whether the user has
  // specified system name/system group or not.

  if (systemName.length() == 0)
    if (systemGroupName.length() == 0)
      service = new FmcjExecutionService(); // Take values from profile
    else
      service = new FmcjExecutionService(systemGroupName);
  else
    service = new FmcjExecutionService(systemName, systemGroupName);

/*-------------------------------------------------------------------*/
/* Logon to the specified system. The 'Default' session mode allows  */
/* multiple logons of the same user, but prevents from participation */
/* in workitem pushing. This is only possible in session mode        */
/* 'Present'. A user can have only one 'Present' session. The        */
/* parameter 'PresentHere' is used to force the closing of the       */
/* user's previous 'Present' session to allow opening a new one.     */
/*-------------------------------------------------------------------*/
  rc = service->Logon( user, password, sessionMode );

  if (rc != FMC_OK)
  {
    cout << "Logon failed with RC " << rc << " - "
         << result->MessageText() << endl;
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    delete(service);
    return rc;
  }

  cout << "Logged on with UserID '" << service->UserID()
       << "' on system '" << service->SystemName()
       << "' at system group '" << service->SystemGroupName()
       << "'." << endl;

  cout << "Timeout value: " << service->Timeout() << "ms" << endl;
  cout << "Session mode : ";
  switch (sessionMode)
  {
    case FmcjService::Default: cout << "Default" << endl; break;
    case FmcjService::Present: cout << "Present" << endl; break;
    case FmcjService::PresentHere: cout << "PresentHere" << endl; break;
  }


/*-------------------------------------------------------------------*/
/* Query information about the logged-on user.                       */
/*-------------------------------------------------------------------*/

  FmcjPerson            loggedOnPerson;
  vector<string>        personRoles;

  // Fetch information about currently logged-on user

  rc = service->UserSettings ( loggedOnPerson );
  if (rc != FMC_OK)
  {
    cout << "Getting user data failed with RC " << rc << " - "
         << result->MessageText() << endl;
    service->Logoff();
    // Deinit API environment
    FmcjGlobal::Disconnect();
    delete(result);
    delete(service);
    return rc;
  }

  loggedOnPerson.Refresh();

  cout << "\nSome information about logged-on user '" << user
       << "':" << endl;

  cout << "     " << "First name        : " << loggedOnPerson.FirstName() << endl;
  cout << "     " << "Last  name        : " << loggedOnPerson.LastName() << endl;
  cout << "     " << "Level             : " << loggedOnPerson.Level() << endl;
  cout << "     " << "Organization      : " << loggedOnPerson.OrganizationName() << endl;
  cout << "     " << "Administrator flag: " << loggedOnPerson.IsAdministrator() << endl;
  cout << "     " << "Manager flag      : " << loggedOnPerson.IsManager() << endl;

  loggedOnPerson.NamesOfRoles(personRoles);

  cout << "     " << "Member of roles   : ";

  for (c=0; c<personRoles.size(); c++)
  {
    if (c!=(personRoles.size()-1)) cout << personRoles[c] << ", ";
    else                           cout << personRoles[c] << endl;
  }

  if (loggedOnPerson.SubstituteIsNull()) cout << "     Substitute        : <not set>\n";
  else cout << "     " << "Substitute        : " << loggedOnPerson.Substitute() << endl;
  cout << "     " << "Absence flag      : " << loggedOnPerson.IsAbsent() << endl;
  cout << "     " << "IsResetAbsence    : " << loggedOnPerson.IsResetAbsence() << endl;

  // Clean up the string vector
  personRoles.erase(personRoles.begin(),personRoles.end());

/*-------------------------------------------------------------------*/
/* Provide supported actions                                         */
/*-------------------------------------------------------------------*/
  cout << "\nCommands: [p] Change user's password            [a] Toggle user's absence flag\n"
       << "          [s] Set a substitute for this user    [q] Quit program\n"
       << "[p/s/a/q]? ";
  cout.flush();

  cin >> choice;

  if (strncmp(choice.c_str(),"p",1)==0) usrchoice=1;
  if (strncmp(choice.c_str(),"s",1)==0) usrchoice=2;
  if (strncmp(choice.c_str(),"a",1)==0) usrchoice=3;

  string                                pw              ="";
  string                                pwchk           ="";

  switch (usrchoice)
  {
    case 1:
/*-------------------------------------------------------------------*/
/* Change the password for the logged-on user.                       */
/*-------------------------------------------------------------------*/

      cout << "\nEnter new password             : "; cout.flush();
      cin >> pw;
      cout << "Enter it again for verification: "; cout.flush();
      cin >> pwchk;

      if (pw==pwchk)
      {
        cout << "\nChanging password." << endl;
        rc = service->SetPassword(pw);
        if (rc != FMC_OK)
        {
          cout << "Setting password failed with RC " << rc << " - "
               << result->MessageText() << endl;
        }
      }
      else
      {
        cout << "\nPasswords do not match." << endl;
      }

      break;


    case 2:
/*-------------------------------------------------------------------*/
/* Set a substitute for the logged-on user.                          */
/*-------------------------------------------------------------------*/
      cout << "\nEnter substitute user name: ";
      cout.flush();

      cin >> choice;

      rc = loggedOnPerson.SetSubstitute( &choice );
      if (rc != FMC_OK)
      {
        cout << "Setting substitute failed with RC " << rc << " - "
             << result->MessageText() << endl;
      }
      cout << "Substitute successfully set to " << choice << endl;

      break;


    case 3:
/*-------------------------------------------------------------------*/
/* Toggle the user's absence flag                                    */
/* The flag may be reset during next logon depending on the person's */
/* reset absence setting.                                            */
/*-------------------------------------------------------------------*/
      if (loggedOnPerson.IsAbsent()) rc = loggedOnPerson.SetAbsence(false);
      else                           rc = loggedOnPerson.SetAbsence(true);

      if (rc != FMC_OK)
      {
        cout << "Toggling absence flag failed with RC " << rc << " - "
             << result->MessageText() << endl;
      }
      cout << "Absence flag set to " << loggedOnPerson.IsAbsent() << endl;

      break;

  }                                     // end switch

/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  cout << "\nLogging off." << endl;

  rc = service->Logoff();

  delete(service);

  cout << "Deinitializing API environment." << endl;

  rc = FmcjGlobal::Disconnect();

  delete(result);

  return 0;
}
