/*BEGIN COPYRIGHT */
/**********************************************************************/
/*                                                                    */
/*  Licensed Materials - Property of IBM                              */
/*  5697-FM3 5655-A96                                                 */
/*  (C) Copyright IBM Corp. 2001, 2004                                */
/*  US Government Users Restricted Rights - Use, duplication or       */
/*  disclosure restricted by GSA ADP Schedule Contract                */
/*  with IBM Corp.                                                    */
/*                                                                    */
/**********************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/******************************************************************************
*   DISCLAIMER
*   ----------
*   This material contains programming source code for your consideration.
*   These examples have not been thoroughly tested under all conditions.
*   IBM, therefore, cannot guarantee or imply reliability, serviceability,
*   or function of these programs.
*   ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*   WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*   NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*   FOR A PARTICULAR PURPOSE.
******************************************************************************/
/* END DISCLAIMER */



/*****************************************************************************/
/* What is this ?                                                            */
/*                                                                           */
/* This is a sample source code demonstrating the way an authentication exit */
/* for MQSeries Workflow operates.                                           */
/* A logon request will only go to the authentication exit if                */
/* it uses the LogonWithCredentials() call from the API. Legacy Logon() calls*/
/* can NOT be handled by the authentication exit.                            */
/* This sample will not connect to any authentication system or read a table */
/* of userID/password pairs, but it will show the basic principles in hard-  */
/* coded scenarios. The scenarios use cleartext passwords as user credentials*/
/* which is NOT RECOMMENDED and VERY INSECURE - but easier to understand.    */
/*                                                                           */
/* To run this example, do the following steps:                              */
/* 1.) compile this code                                                     */
/*     (fmcaexit.dll, libfmcaexit.so or FMCAEXIT, depending on your platform)*/
/* 2.) make it accessible for fmcamain (e.g. same PATH or same directory)    */
/* 3.) Set the profile value as described in the programming guide           */
/* 4.) create a small client program for logging on users with the new       */
/*     LogonWithCredentials() API function.                                  */
/* 5.) Start your client program and issue some logon requests with the      */
/*     user IDs and credentials you find in the examples below.              */
/*     The authentication exit will react differently depending on the user  */
/*     name you use.                                                         */
/*                                                                           */
/*                                                                           */
/* Scenario 1:                                                               */
/*   User: "Administrator"                                                   */
/*   Password: "ASpecialPassword"                                            */
/*   Target mqwfUserID: "ADMIN"                                              */
/*                                                                           */
/*   "Administrator" can log on using the password: "ASpecialPassword".      */
/*   The exit will look up the mqwfUserID "ADMIN" for this user and log on   */
/*   "ADMIN" at the MQWorkflow system.                                       */
/*   Using an other password than "ASpecialPassword" will result in denial of*/
/*   the logon. Note that mqwfUserIDs are all uppercase.                     */
/*                                                                           */
/*                                                                           */
/* Scenario 2:                                                               */
/*   User: "Zaphod Beeblebrox"                                               */
/*   Password: (any)                                                         */
/*   Target mqwfUserID: (none)                                               */
/*                                                                           */
/*   "Zaphod Beeblebrox" can log on using any password, because he will be   */
/*   rejected based on his username. Maybe, this user has left the company,  */
/*   so he is not allowed to connect, even with a correct password.          */
/*                                                                           */
/*                                                                           */
/* Scenario 3:                                                               */
/*   User: "Ford Prefect"                                                    */
/*   Password: "CorrectPassword"                                             */
/*   Target mqwfUserID: "ZYUWLM"                                             */
/*                                                                           */
/*   "Ford Prefect" can logon using any password, too. BUT the lookup for a  */
/*   valid mqwfUserID will return "ZYUWLM". Assuming this is NO valid        */
/*   mqwfUserID (i.e. not defined in any Workflow System), logging on this   */
/*   ID at the Workflow System will result in an unknown user error.         */
/*                                                                           */
/*                                                                           */
/* Scenario 4:                                                               */
/*   User: "Marvin"                                                          */
/*   Password: "recoverable" or "non-recoverable"                            */
/*   Target mqwfUserID: (none)                                               */
/*                                                                           */
/*   This scenario will show you what happens when returning                 */
/*   FMC_EXIT_RECOVERABLE_ERROR or FMC_EXIT_NONRECOVERABLE_ERROR as return   */
/*   value of the "Authenticate" call.                                       */
/*                                                                           */
/*                                                                           */
/* Scenario 5:                                                               */
/*                                                                           */
/*   In this scenario, we will only use credentials to authenticate the user.*/
/*   ( like crypted fingerprints instead of user/password pair. )            */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/* Keep in mind that any failure during authentication should result in      */
/* a denial of the logon.                                                    */
/*                                                                           */
/*****************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <fmcxexit.h>  /* return codes from the authentication exit entry    */
                       /* points: Init() and Deinit()                        */
                       /* - FMC_EXIT_OK                                      */
                       /* - FMC_EXIT_RECOVERABLE_ERROR                       */
                       /* - FMC_EXIT_NONRECOVERABLE_ERROR                    */

#include <fmcaexit.h>  /* authentication exit entry points and return codes  */


/*****************************************************************************/
/* add own includes as needed by your code here                              */
/*****************************************************************************/

/*****************************************************************************/
/* Authentication exit entry point: Init                                     */
/*                                                                           */
/* initialize and build up environment for the authentication exit           */
/*                                                                           */
/* Parameters are reserved. Do not alter them.                               */
/*                                                                           */
/* Return codes                                                              */
/*                                                                           */
/* FMC_EXIT_OK                        - initialization successful            */
/* FMC_EXIT_RECOVERABLE_ERROR         - initialization failed, but exit      */
/*                                      used for further authentications.    */
/* FMC_EXIT_NONRECOVERABLE_ERROR      - initialization failed, exit          */
/*                                      cannot be used for further           */
/*                                      authentications                      */
/*                                                                           */
/* FMC_EXIT_RECOVERABLE_ERROR and FMC_EXIT_NONRECOVERABLE_ERROR will cause   */
/* the administration server to reject any LogonWithCredentials() call.      */
/* However, the administration server will start up under any condition.     */
/*                                                                           */
/*****************************************************************************/
#ifdef __cplusplus
extern "C"
#endif
long FMC_EXPORT
Init ( void ** exitHandle,
            void *  initializationParameter,
            long    initializationParameterLength,
            char *  errorIdBuffer,
            long *  errorIdBufferLength,
            char *  errorDescriptionBuffer,
            long *  errorDescriptionBufferLength )
{
  printf("\nC Authentication exit initializing.");

  /* Do some initialization here if you like                                 */

  printf("\nC Authentication exit is up and running.\n\n");

  /* It is important to return FMC_EXIT_OK when you leave the implementation */
  /* of Init() empty.                                                        */
  return FMC_EXIT_OK;
}



/*****************************************************************************/
/* Authentication exit entry point: Authenticate                             */
/*                                                                           */
/* Authenticate a user against an authentication mechanism with the          */
/* credentials passed by the caller.                                         */
/*                                                                           */
/* Parameters are reserved. Do not alter them.                               */
/*                                                                           */
/* Return codes                                                              */
/*                                                                           */
/* FMC_EXIT_OK                   - Authentication successful                 */
/* FMC_EXIT_RECOVERABLE_ERROR    - Authentication failed, exit can be        */
/*                                 used for further authentications;         */
/*                                 should be returned if an error occurs     */
/*                                 within the authentication                 */
/*                                 system itself                             */
/* FMC_EXIT_NONRECOVERABLE_ERROR - Authentication failed, exit cannot        */
/*                                 be used for further authentications;      */
/*                                 Returning this will cause fmcamain to shut*/
/*                                 down (which will stop the whole system on */
/*                                 the distributed platforms.)               */
/*                                 Attached clients will wait until their    */
/*                                 limit has been reached.                   */
/*                                                                           */
/*****************************************************************************/
#ifdef __cplusplus
extern "C"
#endif

long FMC_EXPORT
Authenticate( FmcjServerAuthExitAuthenticate *pAuthStruct )
{
  /* The credentials of "Administrator":                                     */
  /* We need to compare the userCredenatials as hex values, because on z/OS, */
  /* comparing userCredentials with char[] = "ASpecialPassword" would        */
  /* fail because there is (of course) no ASCII->EBCDIC conversion of the    */
  /* userCredentials.                                                        */

  /* "ASpecialPassword" (hex, without the quotes (") ) =                     */
  char AdministratorCredentials[16]={ 0x41,0x53,0x70,0x65,0x63,0x69,0x61,0x6c
                                     ,0x50,0x61,0x73,0x73,0x77,0x6f,0x72,0x64};

  /* "CorrectPassword" (hex, without the quotes (") ) =                      */
  char FordPrefectCredentials[15]={ 0x43,0x6f,0x72,0x72,0x65,0x63,0x74,0x50
                                   ,0x61,0x73,0x73,0x77,0x6f,0x72,0x64};

  /* "recoverable" (hex, without the quotes (") ) =                          */
  char MarvinCredentials[11]={ 0x72,0x65,0x63,0x6f,0x76,0x65,0x72,0x61,0x62
                              ,0x6c,0x65};

  /* "credentials" (hex, without the quotes (") ) =                          */
    char NoNameCredentials[11]={ 0x63,0x72,0x65,0x64,0x65,0x6E,0x74,0x69,0x61
                                ,0x6c,0x73};


  /* Values you may need:                                                    */
  /* Parameter list version:       pAuthStruct->exitParameterListVersion     */
  /* Parameter list length:        pAuthStruct->exitParameterListLength      */
  /* Client version:               pAuthStruct->version                      */
  /* Client release:               pAuthStruct->release                      */
  /* Client modlevel:              pAuthStruct->modlevel                     */
  /* User credentials:             pAuthStruct->userCredentials              */
  /* User credentials size:        pAuthStruct->userCredentialsSize          */
  /* Authentication result:        pAuthStruct->exitResult                   */
  /* User name:                    pAuthStruct->userName                     */
  /* MQWF user id:                 pAuthStruct->mqwfUserID                   */
  /* Reason code:                  pAuthStruct->reasonCode                   */


  /* Print out the information coming from the administration server         */
  fprintf( stdout, "phaseNumber = #%d#\n",pAuthStruct->phaseNumber);
  fprintf( stdout, "exitParameterListVersion = #%d#\n"
           ,pAuthStruct->exitParameterListVersion
         );
  fprintf( stdout, "userName= #%s#\n",pAuthStruct->userName);
  fprintf( stdout, "userCredentials= #%.*s#\n",
           pAuthStruct->userCredentialsSize, pAuthStruct->userCredentials);
  fprintf( stdout, "Now starting authentication\n");

  /* Each authentication request will only be accepted under the following   */
  /* conditions:                                                             */
  /* 1) The parameter List MUST have version "2"                             */
  /* 2) This MUST be the first phase in authentication.                      */
  if(    pAuthStruct->exitParameterListVersion == 2
      && pAuthStruct->phaseNumber == 0
    )
  {
    fprintf( stdout, "ParameterListVersion == 0 and phaseNumber == 0. OK.\n");

    fprintf(  stdout, "userCredentialsSize == %u\n"
             ,pAuthStruct->userCredentialsSize
           );

    fprintf(  stdout, "userCredentialsBufferSize == %u\n"
             ,pAuthStruct->userCredentialsBufferSize
           );


    /* Scenario 1:                                                           */
    /*                                                                       */
    /* The logon of "Administrator" will be accepted and mapped to the       */
    /* MQWFUserID "ADMIN", when there are correct userCredentials passed in. */
    /*                                                                       */
    /* Scenario: Different usernames/passwords for user authentication and   */
    /*           MQSeries Workflow system.                                   */
    /*                                                                       */
    if( 0 == strncmp( pAuthStruct->userName
                      ,"Administrator"
                      ,sizeof("Administrator")
                    )
      )
    {
      fprintf( stdout, "\"Administrator\" has been verified and accepted.\n");

      if (  0 == strncmp( (const char *)pAuthStruct->userCredentials
                          ,AdministratorCredentials
                          ,16
                        )
         )
      {
        fprintf( stdout, "UserCredentials have been verified and accepted.\n");

        /* Prepare the authentication exit struct for passing it back:       */
        /* Tell the administration server via exitResult that authentication */
        /* was successful and provide in mqwfUserID a valid MQWorkflow user  */
        /* id to logon at the workflow system.                               */
        pAuthStruct->exitResult = Fmc_Auth_LogonAccepted;
        fprintf( stdout, "Setting mqwfUserID to \"ADMIN\"\n");
        strncpy(pAuthStruct->mqwfUserID, "ADMIN" , sizeof("ADMIN"));
        pAuthStruct->reasonCode = 1;
      }
      else
      {
        fprintf( stdout, "UserCredentials failed the check.\n");

        /* Prepare the struct for passing it back:                           */
        /* Tell the administration server via exitResult that authentication */
        /* was not successful. The administration server will send back a    */
        /* negative response to the caller.                                  */
        pAuthStruct->exitResult = Fmc_Auth_LogonDenied;
        pAuthStruct->reasonCode = 2;
      }
    }



    /* Scenario 2:                                                           */
    /*                                                                       */
    /* "Zaphod Beeblebrox" will be denied by the authentication exit even    */
    /* without checking his password.                                        */
    /*                                                                       */
    /* Scenario: The user is no longer allowed to logon to this system.      */
    /*                                                                       */
    else if ( 0 == strncmp( pAuthStruct->userName
                            ,"Zaphod Beeblebrox"
                            ,sizeof("Zaphod Beeblebrox")
                          )
            )
    {
      fprintf( stdout, "\"Zaphod Beeblebrox\" is not allowed to log in.\n");

      /* prepare the struct for passing it back:                             */
      /* Tell the Administration Server via exitResult that authentication   */
      /* was not successful. The Administration Server will send back a      */
      /* negative response to the caller.                                    */
      pAuthStruct->exitResult = Fmc_Auth_LogonDenied;
      pAuthStruct->reasonCode = 3;
    }



    /* Scenario 3:                                                           */
    /*                                                                       */
    /* This account will be accepted, but an unknown MQWF UserID will be     */
    /* passed to the Admin server                                            */
    /*                                                                       */
    /* Scenario: An outdated MQSeries Workflow username is used accidentally */
    /*                                                                       */
    else if ( 0 == strncmp( pAuthStruct->userName
                            ,"Ford Prefect"
                            ,sizeof("Ford Prefect")
                          )
            )
    {
      fprintf( stdout, "\"Ford Prefect\" will have a non-MQWF user ID.\n");

      if (0 == strncmp( (const char *)pAuthStruct->userCredentials
                        ,FordPrefectCredentials
                        ,15
                      )
         )
      {
        fprintf( stdout, "UserCredentials have been verified and accepted.\n");

        /* Prepare the authentication exit struct for passing it back:       */
        /* Tell the administration server via exitResult that authentication */
        /* was successful and provide in mqwfUserID a MQWorkflow user id     */
        /* to logon at the workflow system WHICH IS NOT VALID.               */
        /* The administration server will try to logon that user but send a  */
        /* negative response to the caller.                                  */
        pAuthStruct->exitResult = Fmc_Auth_LogonAccepted;

        fprintf( stdout, "Setting mqwfUserID to \"zyuwlm\"\n");
        strncpy(pAuthStruct->mqwfUserID, "zyuwlm" , sizeof("zyuwlm"));
        pAuthStruct->reasonCode = 4;
      }
      else
      {
        fprintf( stdout, "User credentials are incorrect.\n");

        /* prepare the struct for passing it back ... we saw this already.   */
        pAuthStruct->exitResult = Fmc_Auth_LogonDenied;
        pAuthStruct->reasonCode = 5;
      }
    }



    /* Scenario 4:                                                           */
    /*                                                                       */
    /* "Marvin" will produce a recoverable or a non-recoverable error        */
    /* depending on the password of the user.                                */
    /*                                                                       */
    /* Scenario: Some REALLY NASTY problems with your authentication service.*/
    /*                                                                       */
    else if (0 == strncmp( pAuthStruct->userName, "Marvin" ,sizeof("Marvin")))
    {
      fprintf( stdout, "\"Marvin\" will produce exit failures.\n");

      if (0 == strncmp( (const char *)pAuthStruct->userCredentials
                        ,MarvinCredentials
                        ,11
                      )
         )
      {
        fprintf( stdout, "Producing recoverable error.\n");

        return FMC_EXIT_RECOVERABLE_ERROR;
      }
      else
      {
        fprintf( stdout, "Producing nonrecoverable error.\n");

        return FMC_EXIT_NONRECOVERABLE_ERROR;
      }
    }

    /* Scenario 5:                                                           */
    /*                                                                       */
    /* This account will only use credentials to authenticate the user.      */
    /* Scenario: Crypted fingerprints instead of user/password pair.         */
    /*                                                                       */
    else if (0 == strncmp( (const char *)pAuthStruct->userCredentials
                           ,NoNameCredentials
                           ,11
                         )
            )
    {
      fprintf( stdout, "This is a user identified by his credentials.\n");
      fprintf( stdout, "UserCredentials have been verified and accepted.\n");

      /* prepare the authentication exit struct for passing it back ...      */
      pAuthStruct->exitResult = Fmc_Auth_LogonAccepted;

      fprintf( stdout, "Setting mqwfUserID to \"ADMIN\"\n");
      strncpy(pAuthStruct->mqwfUserID, "ADMIN" , sizeof("ADMIN"));
      pAuthStruct->reasonCode = 6;
    }



    /* Deny access. Should be the default behavior.                          */
    /*                                                                       */
    /* Scenario: Any unhandled condition: Deny access to the system.         */
    /*                                                                       */
    else
    {
      fprintf( stdout, "Login name did not match authentication. DENIED.\n");

      /* prepare the struct for passing it back. Yawn.                       */
      pAuthStruct->exitResult = Fmc_Auth_LogonDenied;
      pAuthStruct->reasonCode = 99;
    }
  }


  /* This is the place we come to if the parameterListVersion does not match */
  /* or we are not in the first phase (number 0) of the authentication.      */
  /* At this stage, authentication only works in one phase, so do not expect */
  /* the phaseCount to change.                                               */
  /*                                                                         */
  /* Deny access. Should be the default behavior.                            */
  /*                                                                         */
  /* Scenario: Any unhandled condition has to deny access to the system.     */
  /*                                                                         */
  else
  {
    fprintf( stdout, "Authentication impossible. Default behavior: DENY.\n");

    /* prepare the struct for passing it back... */
    pAuthStruct->exitResult = Fmc_Auth_LogonDenied;
    pAuthStruct->reasonCode = 99;
  }

  fprintf( stdout, "These are the results directly from the exit:\n");
  fprintf( stdout, "exitResult = #%d#\n",pAuthStruct->exitResult);
  fprintf( stdout, "MQWFUserID = #%s#\n",pAuthStruct->mqwfUserID);
  fprintf( stdout, "\n");

  /* Even if a logon has been denied: the EXIT ITSELF worked just fine.      */
  /* Returning any other value will result in the Administration server      */
  /* denying the logon attempt or shutting down. See scenario "Marvin".      */
  return FMC_EXIT_OK;
}



/*****************************************************************************/
/* Authentication exit entry point: Deinit                                   */
/*                                                                           */
/* Deinitialize environment of the authentication exit.                      */
/*                                                                           */
/* Parameters are reserved. Do not alter them.                               */
/*                                                                           */
/* Return codes                                                              */
/*                                                                           */
/* FMC_EXIT_OK       - the deinitialization was succesful.                   */
/*                     Do not return any other values, as they won't be      */
/*                     handled anyway.                                       */
/*****************************************************************************/
#ifdef __cplusplus
extern "C"
#endif
long FMC_EXPORT
Deinit (void ** exitHandle,
        char *  errorIdBuffer,
        long *  errorIdBufferLength,
        char *  errorDescriptionBuffer,
        long *  errorDescriptionBufferLength )
{
  printf("\nC Authentication exit is de-initializing.");

  /* Do some deinitialization here if you like */

  printf("\nC Authentication exit is stopped.\n\n");

  /* It is important to return FMC_EXIT_OK when you leave the implementation */
  /* of Init() empty.                                                        */
  return(FMC_EXIT_OK);
}

