/*BEGIN COPYRIGHT */
/**************************************************************************
*
*   Licensed Materials - Property of IBM
*   5697-FM3 5655-A96
*   (C) Copyright IBM Corp. 1998, 2000
*   US Government Users Restricted Rights - Use, duplication or
*   disclosure restricted by GSA ADP Schedule Contract
*   with IBM Corp.
*
**************************************************************************/
/*END COPYRIGHT */

/* BEGIN DISCLAIMER */
/**********************************************************************
*  DISCLAIMER
*  ----------
*  This material contains programming source code for your consideration.
*  These examples have not been thoroughly tested under all conditions.
*  IBM, therefore, cannot guarantee or imply reliability, serviceability,
*  or function of these programs.
*  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
*  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
*  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
*  FOR A PARTICULAR PURPOSE.
**********************************************************************/
/* END DISCLAIMER */

/**********************************************************************/
/* MQSeries Workflow API Examples - Basic functions                   */
/*                                                                    */
/* This sample code shows how to establish a session to a Workflow    */
/* server and query details about the logged-on user.                 */
/* Additionally, it can be used to change a user's password, handle   */
/* its substitute and toggle its absence status.                      */
/**********************************************************************/

import com.ibm.workflow.api.*;
import com.ibm.workflow.api.ServicePackage.*;
import java.io.*;

public class BasicFunctions {

public static void main (String[] args)
{
  int c, i = 0; // loop variables

/*--------------------------------------------------------------------*/
/* Check command line parameters                                      */
/*--------------------------------------------------------------------*/

  if ( (args.length<4) || (args.length>7) )
  {
    System.out.println("\nUsage: java BasicFunctions " +
                       " [-p|-pf] <name of agent> <loc | jndi> <UserID> <Password> [System Group] [System Name]\n\n" +
                       "       -p : Session mode 'present'\n" +
                       "       -pf: Force logon for session mode 'present'\n");
    System.exit(-1);
  }

  // Check for session mode option

  SessionMode   sessionMode = SessionMode.DEFAULT;

  boolean ignoredOption = false;

  if (args[0].equalsIgnoreCase("-pf")) sessionMode=SessionMode.PRESENT_HERE;
  else if (args[0].equalsIgnoreCase("-p")) sessionMode=SessionMode.PRESENT;
  else if (args[0].startsWith("-"))
       { System.out.println("\nIgnoring unknown option: " + args[0]);
         ignoredOption = true; }

  // Fetch the correct parameters depending on whether an option was given

  if (sessionMode==SessionMode.DEFAULT && !ignoredOption) i=0; else i=1;

  String user     = args[i+2];
  String password = args[i+3];


  try { // general try clause for whole example


  // Initialize agent

  Agent agent = new Agent();

  if      (args[i+1].equalsIgnoreCase("loc"))  agent.setLocator(Agent.LOC_LOCATOR);
  else if (args[i+1].equalsIgnoreCase("jndi")) agent.setLocator(Agent.JNDI_LOCATOR);
  else
  {
    System.out.println("Invalid locator policy: " + args[i+1]);
    System.exit(-1);
  }

  agent.setName(args[i]);

  // Initialize execution service

  String systemGroupName="";
  String systemName="";

  if (args.length>(i+4))
  {
    if   (args.length==(i+5)) systemGroupName=args[i+4];
    else { systemGroupName=args[i+4]; systemName=args[i+5]; }
  }

  ExecutionService service = agent.locate(systemGroupName,systemName);

  System.out.println();


/*-------------------------------------------------------------------*/
/* Logon to the specified system. The 'Default' session mode allows  */
/* multiple logons of the same user, but prevents from participation */
/* in workitem pushing. This is only possible in session mode        */
/* 'Present'. A user can have only one 'Present' session. The        */
/* parameter 'PresentHere' is used to force the closing of the       */
/* user's previous 'Present' session to allow opening a new one.     */
/*-------------------------------------------------------------------*/

  service.logon2( user, password, sessionMode, AbsenceIndicator.LEAVE );

  System.out.println("Logged on with UserID '" + service.userID() +
                     "' on system '" + service.systemName() +
                     "' at system group '" + service.systemGroupName() +"'.");

  System.out.println("Timeout value: " + service.timeout() + "ms");
  System.out.print("Session mode : ");
  if (sessionMode==SessionMode.DEFAULT) System.out.println("Default");
  if (sessionMode==SessionMode.PRESENT) System.out.println("Present");
  if (sessionMode==SessionMode.PRESENT_HERE) System.out.println("PresentHere");


/*-------------------------------------------------------------------*/
/* Query information about the logged-on user.                       */
/*-------------------------------------------------------------------*/

  // Fetch information about currently logged-on user

  Person loggedOnPerson = service.userSettings();

  loggedOnPerson.refresh();

  System.out.println("\nSome information about logged-on user '" + user + "':");

  System.out.println("     " + "First name        : " + loggedOnPerson.firstName());
  System.out.println("     " + "Last  name        : " + loggedOnPerson.lastName());
  System.out.println("     " + "Level             : " + loggedOnPerson.level());
  System.out.println("     " + "Organization      : " + loggedOnPerson.organizationName());
  System.out.println("     " + "Administrator flag: " + loggedOnPerson.isAdministrator());
  System.out.println("     " + "Manager flag      : " + loggedOnPerson.isManager());

  String personRoles[] = loggedOnPerson.namesOfRoles();

  System.out.print("     " + "Member of roles   : ");

  for (c=0; c<personRoles.length; c++)
  {
    if (c!=(personRoles.length-1)) System.out.print(personRoles[c] + ", ");
    else                           System.out.println(personRoles[c]);
  }

  System.out.println("     " + "Substitute        : " + loggedOnPerson.substitute());
  System.out.println("     " + "Absence flag      : " + loggedOnPerson.isAbsent());


/*-------------------------------------------------------------------*/
/* Query user for action                                             */
/*-------------------------------------------------------------------*/

  System.out.print("\nCommands: [p] Change user's password            [a] Toggle user's absence flag\n" +
                     "          [s] Set a substitute for this user    [q] Quit program\n" +
                     "[p/s/a/q]? ");

  BufferedReader in = new BufferedReader(new InputStreamReader(System.in));

  String choice = in.readLine();


/*-------------------------------------------------------------------*/
/* Change the password for the logged-on user.                       */
/*-------------------------------------------------------------------*/
  if (choice.equalsIgnoreCase("p"))
  {

    String pw="";
    String pwchk="";

    System.out.print("\nEnter new password             : ");
    pw = in.readLine();
    System.out.print("Enter it again for verification: ");
    pwchk = in.readLine();

    if (pw.equals(pwchk))
    {
      System.out.println("\nChanging password.");
      service.setPassword(pw);
    }
    else System.out.println("\nPasswords do not match.");

  }


/*-------------------------------------------------------------------*/
/* Set a substitute for the logged-on user.                          */
/*-------------------------------------------------------------------*/

  if (choice.equalsIgnoreCase("s"))
  {
    System.out.print("\nEnter substitute user name: ");
    choice = in.readLine();

    loggedOnPerson.setSubstitute( choice );
  }


/*-------------------------------------------------------------------*/
/* Toggle the user's absence flag                                    */
/*-------------------------------------------------------------------*/

  if (choice.equalsIgnoreCase("a"))
  {

    if (loggedOnPerson.isAbsent()) loggedOnPerson.setAbsence(false);
    else                           loggedOnPerson.setAbsence(true);

  }


/*-------------------------------------------------------------------*/
/* Logoff and deinit the API environment.                            */
/*-------------------------------------------------------------------*/

  System.out.println("\nLogging off.");

  service.logoff();

  System.exit(0);

  } // end of general try clause

  catch(FmcException e)
  {
    // Catch and report details about the FmcException
    System.out.println("FmcException occured");
    System.out.println("  RC         : " + e.rc);
    System.out.println("  Origin     : " + e.origin);
    System.out.println("  MessageText: " + e.messageText);
    System.out.println("  Exception  : " + e.getMessage());
    System.out.println("  Parameters : ");
    for ( i = 0; i < e.parameters.length ; i++)
    {
      System.out.println("    " + e.parameters[i] );
    }
    System.out.println("  StackTrace : ");
    e.printStackTrace();
  }
  catch(Exception e)
  {
    // Catch and report any exception that occurred.
    e.printStackTrace();
  }

} // end of void main

} // end of BasicFunctions
